'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function createArray(length, map) {
    if (typeof map !== 'function')
        throw new TypeError('"map" is not a function');
    return Array(length).fill(null).map((value, index) => map(index));
}

function moveIndex(array, oldIndex, newIndex) {
    if (!Array.isArray(array)) {
        throw new TypeError('Array to modify must be an array.');
    }
    if (typeof oldIndex !== 'number' || typeof newIndex !== 'number') {
        throw new TypeError('Provided indexes must be a number.');
    }
    if (oldIndex < 0 || newIndex < 0) {
        throw new TypeError('Provided indexes must not be negative.');
    }
    array = array.concat();
    if (newIndex >= array.length) {
        let k = newIndex - array.length + 1;
        while (k--) {
            array.push(null);
        }
    }
    array.splice(newIndex, 0, array.splice(oldIndex, 1)[0]);
    return array;
}

function remove(array, value) {
    if (!Array.isArray(array)) {
        throw new TypeError('Array to modify must be an array.');
    }
    return array.filter((x) => x !== value);
}

function removeAt(array, index) {
    if (!Array.isArray(array)) {
        throw new TypeError('Array to modify must be an array.');
    }
    if (typeof index !== 'number') {
        throw new TypeError('The index to remove must be a number.');
    }
    return array.filter((item, i) => i !== index);
}

function shuffle(array) {
    if (!Array.isArray(array)) {
        throw new TypeError('Array to modify must be an array.');
    }
    var newArray = [];
    var currentIndex = array.length, temporaryValue, randomIndex;
    while (0 !== currentIndex) {
        randomIndex = Math.floor(Math.random() * currentIndex);
        currentIndex -= 1;
        if (newArray.hasOwnProperty(currentIndex)) {
            temporaryValue = newArray[currentIndex];
        }
        else {
            temporaryValue = array[currentIndex];
        }
        newArray[currentIndex] = array[randomIndex];
        newArray[randomIndex] = temporaryValue;
    }
    return newArray;
}

function unique(array) {
    if (!Array.isArray(array)) {
        throw new TypeError('Array to modify must be an array.');
    }
    return [...new Set(array)];
}

exports.createArray = createArray;
exports.moveIndex = moveIndex;
exports.remove = remove;
exports.removeAt = removeAt;
exports.shuffle = shuffle;
exports.unique = unique;
